use std;

use kmeansdata::KmeansData;
use cluster::{Cluster, Clusterable};

pub struct Kmeans<T: Clusterable> {
    pub centroids: Vec<T>,
    pub data: KmeansData<T>,
}

impl<T:Clusterable> Kmeans<T> {
    pub fn new(centroids: Vec<T>, data: Vec<Vec<T>>) -> Kmeans<T> {
        Kmeans {
            centroids: centroids,
            data: KmeansData::from_clusters(data),
        }
    }

    pub fn guess_centroids(data: Vec<Vec<T>>) -> Kmeans<T> {
        let mut centroids = vec![];
        for cluster in &data {
            if let Some(centroid) = T::get_centroid(cluster) {
            centroids.push(centroid);
            }
        }

        Kmeans::new(centroids, data)

    }

    fn from_data(centroids: Vec<T>, data: KmeansData<T>) -> Kmeans<T> {
        Kmeans {
            centroids: centroids,
            data: data,
        }
    }

    pub fn next_iteration(self) -> (Kmeans<T>, bool) {
        let (new_centroids, data) = self.data.iterate(&self.centroids);
        let stable = new_centroids == self.centroids;
        (Kmeans::from_data(new_centroids, data), stable)
    }

    pub fn iter(&self) -> std::slice::Iter<Cluster<T>> {
        self.data.clusters()
    }
}

